"""Info module

This is a module which allows wrapper access to the technical detail
of SpaceX, together with information about the API and applications
using it

This file is imported as a module and contains the following
functions:

    * company - returns company information
    * api - returns information the API version and endpoints etc
    * clients - returns information regarding clients (wrappers)
    * apps - returns applications using the SpaceX API

"""
from . import urldata
from . import utils


def company(timeOut=1):
    """

    :type timeOut: Optional[int]

     Returns details about the SpaceX company

    Parameter
    ----------
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of the SpaceX company

    """
    utils.validateParameters('', __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_info
    return utils.makeRequest(requestUrl, timeOut)


def api(timeOut=1):
    """

    :type timeOut: Optional[int]

     Returns details about the SpaceX API

    Parameter
    ----------
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of the SpaceX API

    """
    utils.validateParameters('', __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_api
    return utils.makeRequest(requestUrl, timeOut)


def clients(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about clients (wrappers) using the SpaceX API

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all clients
        respecting any filters
    """
    utils.validateParameters('', __name__, utils.func_name())
    return utils.getAPISupporting("clients", parameters, timeOut)


def apps(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about applications using the SpaceX API

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all applications
        respecting any filters
    """
    utils.validateParameters('', __name__, utils.func_name())
    return utils.getAPISupporting("apps", parameters, timeOut)



"""Ships module

This is a module which allows wrapper access to the functions within
the SpaceX API to return information about SpaceX fleet of ships.
This includes support and recovery vessels as well as the drone ships

This file is imported as a module and contains the following
functions:

    * ships - returns all ship information
    * one - returns information about a specific ship

"""
from . import urldata
from . import utils


def ships(parameters='', timeOut=1):
    """
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Parameters
    ----------
    parameters
        optional - a JSON document containing valid query modifiers
    timeOut
        optional - an integer stating the timeout in seconds
                    of the REST api call

    Returns
    -------
    string
        a JSON document containing details about all ships
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_ships
    return utils.makeRequest(requestUrl, timeOut, parameters)


def one(ship='', parameters='', timeOut=1):
    """
    :type ship: [str]
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Parameters
    ----------
    ship
        the idenfier of the ship of interest
    parameters
        optional - a JSON document containing valid query modifiers
    timeOut
        optional - an integer stating the timeout in seconds
                    of the REST api call

    Returns
    -------
    string
        a JSON document containing details about a specific ship
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main \
        + urldata.Domain.main_ships + "/" + ship
    return utils.makeRequest(requestUrl, timeOut, parameters)



"""Launchpads module

This is a module which allows wrapper access to the functions within
the SpaceX API to return information about the launchpads that
SpaceX use

This file is imported as a module and contains the following
functions:

    * launchpads - returns all launchpad information
    * one - returns information about a specific launchpad

"""

from . import urldata
from . import utils


def launchpads(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about all launchpads

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all the launchpads
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    try:
        requestUrl = urldata.Domain.main + \
            urldata.Domain.main_launchpads
    except utils.SpaceXReadTimeOut as e:
        raise e
    else:
        return utils.makeRequest(requestUrl, timeOut, parameters)


def one(pad='', parameters='', timeOut=1):
    """

    :type pad: str
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about a specific launchpad

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all the launchpads
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    try:
        requestUrl = urldata.Domain.main + \
            urldata.Domain.main_launchpads + "/" + pad
    except utils.SpaceXReadTimeOut as e:
        raise e
    else:
        return utils.makeRequest(requestUrl, timeOut, parameters)



import sys
sys.path.append('../')

import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *


def test_payloads():
    payloads_data = ''
    payloads_result = keyOrder(
        alphaOrder(readJSONFile('payloads/all.json')), 'payload_id')
    try:
        payloads_data = keyOrder(alphaOrder(
            spacexpython.payloads.payloads('', 1)), 'payload_id')
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on payloads.allpayloads")
    assert payloads_data == payloads_result


@pytest.mark.skip(reason="Skip")
def test_one_payload():
    one_data = ''
    one_result = alphaOrder(
        readJSONFile('payloads/oneTelkom-4.json'))
    try:
        one_data = alphaOrder(
            spacexpython.payloads.one('Telkom-4', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on payloads.one('Telkom-4')")
    assert one_data == one_result




"""Rockets module

This is a module which allows wrapper access to the
functions within the SpaceX API to return information
about the rockets

This file is imported as a module and contains the following
functions:

    * all - returns all rockets
    * rocket - returns information about one rocket rocket type


    * falcon1 - returns information about the Falcon 1
    * falcon9 - returns information about the Falcon 9
    * bfr - returns information about the Big Falcon Rocket *DEPRECATED
    * falconheavy - returns information about the Falcon Heavy
    * starship - returns information about the Starship
"""
import warnings
from . import urldata
from . import utils


def allrockets(parameters='', timeOut=1):
    """ Return a JSON document containing ALL rockets

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Parameters
    ----------
    requestURL : str
        string to pass into the REST API
    timeOut : int
        optional - API call timeout
    parameters : str
        optional parameters to append to URL as query modifiers

    Exceptions
    ----------
    SpaceXReadTimeOut
        an exception raised when the API call breaches the timeout limit

    Returns
    -------
    string
        a JSON document containing a list of rockets
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_rockets
    return utils.makeRequest(requestUrl, timeOut, parameters)


def rocket(rocket, parameters='', timeOut=1):
    """ Return a JSON document containing details of a specific rocket

    :type rocket: str
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Parameters
    ----------
    rocket
        mandatory - the rocket ID of the rocket in question
    parameters
        optional - a JSON document containing valid query modifiers
    timeOut
        optional - an integer stating the timeout in seconds
            of the REST api call

    Exceptions
    ----------
    SpaceXReadTimeOut
        an exception raised when the API call breaches the timeout limit

    Returns
    -------
    string
        a JSON document containing a single rocket
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + \
        urldata.Domain.main_rockets + "/" + rocket
    return utils.makeRequest(requestUrl, timeOut)


def falcon1(parameters='', timeOut=1):
    """ Return a JSON document containing details of the Falcon 1 rocket

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Parameters
    ----------
    parameters
        optional - a JSON document containing valid query modifiers
    timeOut
        optional - an integer stating the timeout in seconds
            of the REST api call

    Exceptions
    ----------
    SpaceXReadTimeOut
        raised when the API call breaches the timeout limit

    Returns
    -------
    string
        a JSON document containing details about the Falcon 1 rocket
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.falcon1
    return utils.makeRequest(requestUrl, timeOut, parameters)


def falcon9(parameters='', timeOut=1):
    """ Return a JSON document containing details of the Falcon 9 rocket

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Parameters
    ----------
    parameters
        optional - a JSON document containing valid query modifiers
    timeOut
        optional - an integer stating the timeout in seconds
            of the REST api call

    Exceptions
    ----------
    SpaceXReadTimeOut
        raised when the API call breaches the timeout limit

    Returns
    -------
    string
        a JSON document containing details about the Falcon 9 rocket
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.falcon9
    return utils.makeRequest(requestUrl, timeOut, parameters)


def falconheavy(parameters='', timeOut=1):
    """ Return a JSON document containing details of
        the Falcon Heavy rocket

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Parameters
    ----------
    parameters
        optional - a JSON document containing valid query modifiers
    timeOut
        optional - an integer stating the timeout in seconds
                of the REST api call

    Exceptions
    ----------
    SpaceXReadTimeOut
        raised when the API call breaches the timeout limit

    Returns
    -------
    string
        a JSON document containing details about the Falcon Heavy rocket
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.falconheavy
    return utils.makeRequest(requestUrl, timeOut, parameters)


def bfr(parameters='', timeOut=1):
    """ Return a JSON document containing details of
        the Big Falcon Rocket

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Parameters
    ----------
    parameters
        optional - a JSON document containing valid query modifiers
    timeOut
        optional - an integer stating the timeout in seconds
             of the REST api call

    Exceptions
    ----------
    SpaceXReadTimeOut
        an exception raised when the API call breaches the timeout limit

    Returns
    -------
    string
        a JSON document containing details about the Big Falcon Rocket
    """
    warnings.warn(
        "spacexpython.rockets.BFR is deprecated use "
        "spacexpython.rockets.starship instead",
        DeprecationWarning
    )
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.bigfalconrocket
    return utils.makeRequest(requestUrl, timeOut, parameters)


def starship(parameters='', timeOut=1):
    """ Return a JSON document containing details of a the Starship

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Parameters
    ----------
    parameters
        optional - a JSON document containing valid query modifiers
    timeOut
        optional - an integer stating the timeout in seconds
                    of the REST api call

    Exceptions
    ----------
    SpaceXReadTimeOut
        raised when the API call breaches the timeout limit

    Returns
    -------
    string
        a JSON document containing details about the Starship
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.starship
    return utils.makeRequest(requestUrl, timeOut, parameters)



"""Cores module

This is a module which allows wrapper access to the functions within
the SpaceX API to return information about the cores

This file is imported as a module and contains the following
functions:

    * cores - returns all cores
    * upcoming - returns information about upcoming cores
    * past - returns information about past cores
    * one - returns information about a specific core

"""
from . import urldata
from . import utils


def cores(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about all cores

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all the cores
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_cores
    return utils.makeRequest(requestUrl, timeOut, parameters)


def upcoming(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about upcoming capsules

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of upcoming capsules
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.upcoming_cores
    return utils.makeRequest(requestUrl, timeOut, parameters)


def past(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about past capsules

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all the past capsules
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.past_cores
    return utils.makeRequest(requestUrl, timeOut, parameters)


def one(core_serial, parameters='', timeOut=1):
    """

    :type core_serial: str
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about a specific core

    Parameters
    ----------
    core_serial : Serial number of the core of interest
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of a specific core
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.main_cores + \
        "/" + str(core_serial)
    return utils.makeRequest(requestUrl, timeOut, parameters)




import sys
sys.path.append('../')
import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *


def test_missions():
    missions_data = ''
    missions_result = keyOrder(
        alphaOrder(readJSONFile('missions/all.json')), 'mission_id')
    try:
        missions_data = keyOrder(alphaOrder(
            spacexpython.missions.missions('', 1)), 'mission_id')
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on missions.allmissions")
    assert missions_data == missions_result


def test_one_mission():
    one_data = ''
    one_result = alphaOrder(
        readJSONFile('missions/oneF3364BF.json'))
    try:
        one_data = alphaOrder(
            spacexpython.missions.one('F3364BF', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on missions.one('F3364BF')")
    assert one_data == one_result



"""Utilities module for the test suite

This is a utilities module with common functionality
which is used by many other modules.

This file is imported as a module and contains the following
functions:

    * readJSONFile - reads a JSON file into a value
    * writeJSONFile - writes a JSON document into a named file
    * keyOrder - orders a JSON document by key (for lists.....)
    * alphaOrder - order a JSON document by alphabetic key name
"""

import json

import requests


def keyOrder(JSONDocument, ky):
    """ Orders a JSON Document by key order

    Parameters
    ----------
    JSONDocument
        unsorted JSON Document
    ky
        key by which the JSON document should be ordered

    Returns
    -------
    response
        the JSON document ordered by key
    """
    return json.loads(JSONDocument).sort(
        key=lambda k: k[ky], reverse=False)


def alphaOrder(JSONDocument):
    """ Orders a JSON Document by alphabetic key

    Parameters
    ----------
    JSONDocument
        unsorted (alphabetically) JSON Document

    Returns
    -------
    response
        the JSON document ordered by alphabetic key
    """
    return json.dumps(JSONDocument, sort_keys=True)


def readJSONFile(filename):
    """ Reads a named JSON file and retrieves the document from it.

    Parameters
    ----------
    filename : str
        name of the file

    Returns
    -------
    response
        the JSON document read from the file
    """
    filename = 'data/' + filename
    with open(filename) as json_file:
        response = json.load(json_file)
    return response


def writeJSONFile(filename, JSONDocument):
    """ Writes a JSON document to a named file

    Parameters
    ----------
    filename : str
        name of the file
    JSONDocument : str
        JSON document to write to the file

    Returns
    -------
    True
    """

    filename = 'data/' + filename
    with open(filename, 'w') as outfile:
        json.dump(JSONDocument, outfile)
    return True


def writeFile(filename, data, mode):
    """ Writes/appends data to a file

    Parameters
    ----------
    filename : str
        name of the file
    data : str
        text to write to the file
    mode: str
        append/write etc
    Returns
    -------
    True
    """

    with open(filename, mode) as outfile:
        outfile.write(data)
    return True


def makeHTTP(requestUrl, timeOut=1):
    """ sends a request to an http(s) endpoint

    Parameters
    ----------
    requestUrl : str
        url to call
    timeOut : int
        optional - call timeout

    Returns
    -------
    response
        a string which is a string returned from the http call

    Exceptions
    ----------
    SpaceXReadTimeOut
        raised when the API call breaches the timeout limit
    """
    try:
        url_response = requests.get(url=str(requestUrl), timeout=timeOut, headers={   # noqa: E501
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36'}  # noqa: E501
                                    )
    except requests.exceptions.ReadTimeout:
        raise TimeoutError('HTTP Timeout Error')
    else:
        response = url_response.text
    return response


def percentage(percent, whole):
    return (percent * whole) / 100.0


def plus_percent(percent, value):
    return value + percentage(percent, value)


def minus_percent(percent, value):
    return value - percentage(percent, value)



"""Exceptions module

This module contains the exceptions which SpacePY-X generates:

This file is imported as a module and contains the following
Exceptions:

    * SpaceXReadTimeOut - if a call to the API times out.
        The timeout respects the timeOut parameter on all functions
    * SpaceXParameterError - if a call to a wrapper function contains
        either invalid parameter names (or types), this exception is
        returned together with a message stating where the issue lies.
"""


class SpaceXReadTimeOut(Exception):
    pass


class SpaceXParameterError(Exception):
    pass



"""Payload module

This is a module which allows wrapper access to the functions within
the SpaceX API to return information about payloads SpaceX has carried

This file is imported as a module and contains the following
functions:

    * payloads - returns all payload information
    * one - returns information about a specific payload

"""
from . import urldata
from . import utils


def payloads(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about payloads SpaceX has carried

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all payloads
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    try:
        requestUrl = urldata.Domain.main + urldata.Domain.main_payloads
    except utils.SpaceXReadTimeOut as e:
        raise e
    else:
        return utils.makeRequest(requestUrl, timeOut, parameters)


def one(payload='', parameters='', timeOut=1):
    """
    :type payload : [str]
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about an individual payload SpaceX has carried

    Parameters
    ----------
    payload : Payload idnentifier (str)
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of a single payload
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    try:
        requestUrl = urldata.Domain.main + \
            urldata.Domain.main_payloads + "/" + payload
    except utils.SpaceXReadTimeOut as e:
        raise e
    else:
        return utils.makeRequest(requestUrl, timeOut, parameters)




import sys
sys.path.append('../')

import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *


def test_dragons():
    dragons_data = ''
    dragons_result = keyOrder(alphaOrder(readJSONFile(
        'dragons/all.json')), 'id')
    try:
        dragons_data = keyOrder(alphaOrder(
            spacexpython.dragons.dragons('', 1)), 'id')
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on dragons.alldragons")
    assert dragons_data == dragons_result


def test_one_dragon():
    one_data = ''
    one_result = alphaOrder(readJSONFile(
        'dragons/onedragon1.json'))
    try:
        one_data = alphaOrder(
            spacexpython.dragons.one('dragon1', '', 1))
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on dragons.one('dragon1")
    assert one_data == one_result



from setuptools import setup, find_packages
import os

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()


install_requires = [{
    'beautifulsoup4',
    'cachepy',
    'requests',
    'tinydb'
    }
]


setup(name='SpacePY-X',
      version="v1.1.2",
      description="Simple Python wrapper for the SpaceX API",
      long_description=README,
      classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
      ],
      keywords='wrapper api spacex python',
      author='Andrew Shapton',
      author_email='alshapton@gmail.com',
      url='https://github.com/alshapton/SpacePY-X',
      license='MIT',
      packages=find_packages('src'),

      package_dir={'': 'src'},
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      entry_points={
        'console_scripts':
            ['spacex-python=spacexpython']
                }
      )




import sys
sys.path.append('../')
import pytest
import spacexpython
from .tutils import *


def test_api():
    api_data = ''
    api_result = alphaOrder(readJSONFile('info/api.json'))
    try:
        api_data = alphaOrder(spacexpython.info.api())
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on info.api")
    assert api_result == api_data


def test_company():
    company_data = ''
    company_result = alphaOrder(readJSONFile('info/company.json'))
    try:
        company_data = alphaOrder(spacexpython.info.company())
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print ("Failure on info.company")
    assert company_data == company_result


def test_clients():
    client_data = ''
    client_result = keyOrder(alphaOrder(
        readJSONFile('info/clients.json')), "Name")
    try:
        client_data = keyOrder(
            alphaOrder(spacexpython.info.clients()), "Name")
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on info.clients")
    assert client_data == client_result


def test_language():
    language_data = ''
    language_result = keyOrder(alphaOrder(
        readJSONFile('info/oneclientmine.json')), "Name")
    try:
        language_data = keyOrder(alphaOrder(
            spacexpython.info.clients(
                '{"Languages":["Python"], '
                + ' "Creators":["Andrew Shapton"]}')), "Name")
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on info.clients(Language)")
    assert language_data == language_result


def test_apps():
    apps_data = ''
    apps_result = keyOrder(alphaOrder(
        readJSONFile('info/apps.json')), "Name")
    try:
        apps_data = keyOrder(
            alphaOrder(spacexpython.info.apps()), "Name")
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on info.apps")
    print(apps_data)
    assert apps_data == apps_result


def test_one_app():
    one_app_data = ''
    one_app_result = keyOrder(alphaOrder(
        readJSONFile('info/oneapsxmw.json')), "Name")
    try:
        one_app_data = keyOrder(alphaOrder(
            spacexpython.info.apps(
                '{"Name":["SpaceX Mission Watch"]}')), "Name")
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on info.apps(Name)")
    assert one_app_data == one_app_result




import sys
sys.path.append('../')

import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *


def test_launches():
    launches_data = ''
    launches_result = keyOrder(alphaOrder(
        readJSONFile('launches/all.json')), 'flight_number')
    try:
        launches_data = keyOrder(alphaOrder(
            spacexpython.launches.alllaunches('', 1)), 'flight_number')
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on launches.all")
    assert launches_data == launches_result



import sys
sys.path.append('../')
import pytest
import spacexpython
from spacexpython.exceptions import *
from spacexpython.utils import *
from .tutils import *
import subprocess
from time import gmtime, strftime
from datetime import timedelta, datetime

# Conversion factors:
# Kilometers to Miles
# Astronomical Units to Kilometres
KM_TO_MILES = 0.621371
AU_TO_KM = 149598073.0

# Date strings
NOW = strftime("%Y-%m-%d %H:%M:%S", gmtime())
END = datetime.now() + timedelta(days=1)
TOMORROW = END.strftime('%Y-%m-%d %H:%M:%S')

# Base directory for API-sourced data
BASE = 'data/roadster/roadster.'


def test_name(setup_module):
    assert json.loads(setup_module)["API_name"] \
        == "Elon Musk\'s Tesla Roadster"


def test_launch_date_utc(setup_module):
    assert json.loads(setup_module)["API_launch_date_utc"] \
        == "2018-02-06T20:45:00.000Z"


def test_launch_date_unix(setup_module):
    assert json.loads(setup_module)["API_launch_date_unix"] \
        == 1517949900


def test_launch_mass_kg(setup_module):
    assert json.loads(setup_module)["API_launch_mass_kg"] \
        == 1350


def test_launch_mass_lbs(setup_module):
    assert json.loads(setup_module)["API_launch_mass_lbs"] \
        == 2976


def test_norad_id(setup_module):
    assert json.loads(setup_module)["API_norad_id"] \
        == 43205


def test_orbit_type(setup_module):
    assert json.loads(setup_module)["API_orbit_type"] \
        == "heliocentric"


def test_wikipedia(setup_module):
    assert json.loads(setup_module)["API_wikipedia"] \
        == "https://en.wikipedia.org/wiki/Elon_Musk%27s_Tesla_Roadster"


def test_details(setup_module):
    assert json.loads(setup_module)["API_details"] == "Elon Musk's " \
        + "Tesla Roadster is an electric sports car that served as " \
        + "the dummy payload for the February 2018 Falcon Heavy " \
        + "test flight and is now an artificial satellite of the " \
        + "Sun. Starman, a mannequin " \
        + "dressed in a spacesuit, occupies the driver's seat. " \
        + "The car and rocket are products of Tesla and SpaceX. " \
        + "This 2008-model Roadster was previously used by Musk for " \
        + "commuting, and is the only consumer car sent into space."


def test_mars_distance_km(setup_module):
    assert minus_percent(1, json.loads(
        setup_module)["LCL_mars_distance_km"]) <= \
        json.loads(
        setup_module)["API_mars_distance_km"] <= \
        plus_percent(1, json.loads(
            setup_module)["LCL_mars_distance_km"])


def test_mars_distance_mi(setup_module):
    assert minus_percent(1, json.loads(
        setup_module)["LCL_mars_distance_mi"]) \
        <= json.loads(setup_module)["API_mars_distance_mi"] <= \
        plus_percent(1, json.loads(
            setup_module)["LCL_mars_distance_mi"])


def test_earth_distance_km(setup_module):
    assert minus_percent(1, json.loads(
        setup_module)["LCL_earth_distance_km"]) <= \
        json.loads(setup_module)["API_earth_distance_km"] <= \
        plus_percent(1, json.loads(
            setup_module)["LCL_earth_distance_km"])


def test_earth_distance_mi(setup_module):
    assert minus_percent(1, json.loads(
        setup_module)["LCL_earth_distance_mi"]) <= \
        json.loads(setup_module)["API_earth_distance_mi"] <= \
        plus_percent(1, json.loads(
            setup_module)["LCL_earth_distance_mi"])


def test_orbital_speed_kph(setup_module):
    assert minus_percent(1, json.loads(
        setup_module)["LCL_speed_kph"]) <= \
        json.loads(setup_module)["API_speed_kph"] <= \
        plus_percent(1, json.loads(
            setup_module)["LCL_speed_kph"])


def test_orbital_speed_mph(setup_module):
    assert minus_percent(1, json.loads(
        setup_module)["LCL_speed_mph"]) <= \
        json.loads(setup_module)["API_speed_mph"] <= \
        plus_percent(1, json.loads(
            setup_module)["LCL_speed_mph"])


def test_epoch(setup_module):
    assert minus_percent(1, json.loads(setup_module)["LCL_epoch"]) <= \
        json.loads(setup_module)["API_epoch"] <= \
        plus_percent(1, json.loads(setup_module)["LCL_epoch"])


@pytest.mark.skip("Skip sma test - variable data (time sensitive)")
def test_sma(setup_module):
    assert minus_percent(1, json.loads(
        setup_module)["LCL_sma"]) \
        <= json.loads(setup_module)["API_sma"] <= \
        plus_percent(1, json.loads(
            setup_module)["LCL_sma"])


def test_ec(setup_module):
    assert minus_percent(1, json.loads(
        setup_module)["LCL_ec"]) \
        <= json.loads(setup_module)["API_ec"] <= \
        plus_percent(1, json.loads(
            setup_module)["LCL_ec"])


def test_qr(setup_module):
    assert minus_percent(1, json.loads(setup_module)["LCL_qr"]) <= \
        json.loads(setup_module)["API_qr"] <= \
        plus_percent(1, json.loads(setup_module)["LCL_qr"])


def test_ad(setup_module):
    assert minus_percent(1, json.loads(setup_module)["LCL_ad"]) <= \
        json.loads(setup_module)["API_ad"] <= \
        plus_percent(1, json.loads(setup_module)["LCL_ad"])


def test_om(setup_module):
    assert minus_percent(1, json.loads(setup_module)["LCL_om"]) <= \
        json.loads(setup_module)["API_om"] <= \
        plus_percent(1, json.loads(setup_module)["LCL_om"])


def test_w(setup_module):
    assert minus_percent(1, json.loads(setup_module)["LCL_w"]) <= \
        json.loads(setup_module)["API_w"] <= \
        plus_percent(1, json.loads(setup_module)["LCL_w"])


def test_inc(setup_module):
    assert minus_percent(1, json.loads(setup_module)["LCL_inc"]) <= \
        json.loads(setup_module)["API_inc"] <= \
        plus_percent(1, json.loads(setup_module)["LCL_inc"])


def test_period(setup_module):
    assert minus_percent(1, json.loads(setup_module)["LCL_period"]) <= \
        json.loads(setup_module)["API_period"] <= \
        plus_percent(1, json.loads(setup_module)["LCL_period"])


@pytest.fixture(scope='module')
def setup_module():
    CDATA = ""
    roadster_data = ''
    try:
        roadster_data = alphaOrder(spacexpython.roadster.roadster())
    except spacexpython.utils.SpaceXReadTimeOut:
        pytest.xfail("Space/X API Read Timed Out")
        print("Failure on info.roadster")
    CDATA = CDATA + '{"API_name":"' \
        + (json.loads(roadster_data)["name"]) + '",'

    CDATA = CDATA + '"API_launch_date_utc":"' \
        + str((json.loads(roadster_data)["launch_date_utc"])) + '",'
    CDATA = CDATA + '"API_launch_date_unix":' \
        + str((json.loads(roadster_data)["launch_date_unix"])) + ','
    CDATA = CDATA + '"API_launch_mass_kg":' \
        + str((json.loads(roadster_data)["launch_mass_kg"])) + ','

    CDATA = CDATA + '"API_launch_mass_lbs":' \
        + str((json.loads(roadster_data)["launch_mass_lbs"])) + ','
    CDATA = CDATA + '"API_norad_id":' \
        + str((json.loads(roadster_data)["norad_id"])) + ','
    CDATA = CDATA + '"API_orbit_type":"' \
        + str((json.loads(roadster_data)["orbit_type"])) + '",'
    CDATA = CDATA + '"API_wikipedia":"' \
        + str((json.loads(roadster_data)["wikipedia"])) + '",'
    CDATA = CDATA + '"API_details":"' \
        + str((json.loads(roadster_data)["details"])) + '",'

    mars_distance_from_api_km = (json.loads(
        roadster_data)["mars_distance_km"])
    mars_distance_from_api_mi = (json.loads(
        roadster_data)["mars_distance_mi"])
    earth_distance_from_api_km = (json.loads(
        roadster_data)["earth_distance_km"])
    earth_distance_from_api_mi = (json.loads(
        roadster_data)["earth_distance_mi"])

    orbital_speed_kph = (json.loads(roadster_data)["speed_kph"])
    orbital_speed_mph = (json.loads(roadster_data)["speed_mph"])

    epoch_from_api = (json.loads(roadster_data)["epoch_jd"])
    sma_from_api = (json.loads(roadster_data)["semi_major_axis_au"])
    ec_from_api = (json.loads(roadster_data)["eccentricity"])
    qr_from_api = (json.loads(roadster_data)["periapsis_au"])
    ad_from_api = (json.loads(roadster_data)["apoapsis_au"])
    om_from_api = (json.loads(roadster_data)["longitude"])
    w_from_api = (json.loads(roadster_data)["periapsis_arg"])
    inc_from_api = (json.loads(roadster_data)["inclination"])
    period_from_api = (json.loads(roadster_data)["period_days"])

    orbitURL = ("https://ssd.jpl.nasa.gov/horizons_batch.cgi?batch=1&COMMAND='-143205'&CENTER= '500@10'&MAKE_EPHEM= 'YES'"                      # noqa: E501
        + "&TABLE_TYPE= 'ELEMENTS'&START_TIME= '" + NOW + "'&STOP_TIME= '" + TOMORROW + "'&STEP_SIZE= '1 d'&OUT_UNITS= 'AU-D'"                  # noqa: E501
        + "&REF_PLANE = 'ECLIPTIC' &REF_SYSTEM = 'J2000' &TP_TYPE = 'ABSOLUTE' &ELEM_LABELS = 'YES' &CSV_FORMAT = 'NO' & OBJ_DATA = 'YES'")    # noqa: E501

    marsDistURL = ("https://ssd.jpl.nasa.gov/horizons_batch.cgi?batch=1&COMMAND='-143205'&CENTER= '500@499'&MAKE_EPHEM= 'YES'"  # noqa: E501
                  + "&TABLE_TYPE= 'OBSERVER'&START_TIME= '" + NOW + "'&STOP_TIME= '" + TOMORROW + "'&STEP_SIZE= '1 d'"          # noqa: E501
                  + "&CAL_FORMAT= 'CAL'&TIME_DIGITS= 'MINUTES'&ANG_FORMAT= 'HMS'&OUT_UNITS= 'KM-S'&RANGE_UNITS= 'AU'"           # noqa: E501
                  + "&APPARENT= 'AIRLESS'&SUPPRESS_RANGE_RATE= 'NO'&SKIP_DAYLT= 'NO'&EXTRA_PREC= 'NO'&R_T_S_ONLY= 'NO'"         # noqa: E501
                  + "&REF_SYSTEM= 'J2000'&CSV_FORMAT= 'NO'&OBJ_DATA= 'YES'&QUANTITIES= '19,20,22'")                            # noqa: E501

    earthDistURL = ("https://ssd.jpl.nasa.gov/horizons_batch.cgi?batch=1&COMMAND='-143205'&CENTER='500@399'"            # noqa: E501
                    + "&MAKE_EPHEM='YES'&TABLE_TYPE='OBSERVER'&START_TIME='" + NOW + "'&STOP_TIME = '" + TOMORROW + "'"  # noqa: E501
                    + "&STEP_SIZE = '1 d' &CAL_FORMAT = 'CAL' &TIME_DIGITS = 'MINUTES' &ANG_FORMAT = 'HMS' "            # noqa: E501
                    + "&OUT_UNITS = 'KM-S' &RANGE_UNITS = 'AU' &APPARENT = 'AIRLESS' &SUPPRESS_RANGE_RATE = 'NO' "      # noqa: E501
                    + "&SKIP_DAYLT = 'NO' &EXTRA_PREC = 'NO' &R_T_S_ONLY = 'NO' &REF_SYSTEM = 'J2000' "                 # noqa: E501
                    + "&CSV_FORMAT = 'NO' &OBJ_DATA = 'YES' &QUANTITIES = '19,20'")                                     # noqa: E501

    # Get the data for the Mars Distance
    # information from JPL Horizons API
    fg = makeHTTP(marsDistURL, 1)

    # Create a new file with the results of the call
    # to the JPL Horizons API for Mars distance
    writeFile(BASE + 'mars', fg, 'w+')

    # Get the data for the Orbit Parameters
    # information from JPL Horizons API
    fg = makeHTTP(orbitURL, 1)

    # Create a new file with the results of the
    # call to the JPL Horizons API for Orbit Parameters
    writeFile(BASE + 'orbit', fg, 'w+')

    # Get the data for the Earth Distance
    # information from JPL Horizons API
    fg = makeHTTP(earthDistURL, 1)

    # Create a new file with the results
    # of the call to the JPL Horizons API for the Earth Distance
    writeFile(BASE + 'earth', fg, 'w+')

    # EPOCH
    sb = ['./script_roadster.zsh epoch', 'epoch']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    epoch = float(g.stdout.strip())

    CDATA = CDATA + '"LCL_epoch":' + str(epoch) + ','
    CDATA = CDATA + '"API_epoch":' + str(epoch_from_api) + ','

    # Semi-major Axis
    sb = ['./script_roadster.zsh sma', 'sma']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    sma = float(g.stdout.strip())
    sma = (sma * float(AU_TO_KM)) / float(10000000)

    CDATA = CDATA + '"LCL_sma":' + str(sma) + ','
    CDATA = CDATA + '"API_sma":' + str(sma_from_api) + ','

    # Eccentricity
    sb = ['./script_roadster.zsh ec', 'ec']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    ec = float(g.stdout.strip())

    CDATA = CDATA + '"LCL_ec":' + str(ec) + ','
    CDATA = CDATA + '"API_ec":' + str(ec_from_api) + ','

    # Periapsis
    sb = ['./script_roadster.zsh qr', 'qr']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    qr = float(g.stdout.strip())

    CDATA = CDATA + '"LCL_qr":' + str(qr) + ','
    CDATA = CDATA + '"API_qr":' + str(qr_from_api) + ','

    # Apoapsis
    sb = ['./script_roadster.zsh ad', 'ad']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    ad = float(g.stdout.strip())

    CDATA = CDATA + '"LCL_ad":' + str(ad) + ','
    CDATA = CDATA + '"API_ad":' + str(ad_from_api) + ','

    # Longitude
    sb = ['./script_roadster.zsh om', 'om']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    om = float(g.stdout.strip())

    CDATA = CDATA + '"LCL_om":' + str(om) + ','
    CDATA = CDATA + '"API_om":' + str(om_from_api) + ','

    # Longitude
    sb = ['./script_roadster.zsh w', 'w']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    w = float(g.stdout.strip())

    CDATA = CDATA + '"LCL_w":' + str(w) + ','
    CDATA = CDATA + '"API_w":' + str(w_from_api) + ','

    # Apoapsis
    sb = ['./script_roadster.zsh inc', 'inc']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    inc = float(g.stdout.strip())

    CDATA = CDATA + '"LCL_inc":' + str(inc) + ','
    CDATA = CDATA + '"API_inc":' + str(inc_from_api) + ','

    # Period (days)
    sb = ['./script_roadster.zsh period', 'period']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    period = float(g.stdout.strip())

    CDATA = CDATA + '"LCL_period":' + str(period) + ','
    CDATA = CDATA + '"API_period":' + str(period_from_api) + ','

    # Distance from Earth
    sb = ['./script_roadster.zsh earthDistance', 'earthDistance']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    distanceFromEarthkm = float(g.stdout.strip()) \
        * float(AU_TO_KM)
    distanceFromEarthmi = float(distanceFromEarthkm) \
        * float(KM_TO_MILES)

    CDATA = CDATA + '"API_earth_distance_km":' \
        + str(earth_distance_from_api_km) + ','
    CDATA = CDATA + '"API_earth_distance_mi":' \
        + str(earth_distance_from_api_mi) + ','
    CDATA = CDATA + '"LCL_earth_distance_km":' \
        + str(distanceFromEarthkm) + ','
    CDATA = CDATA + '"LCL_earth_distance_mi":' \
        + str(distanceFromEarthmi) + ','

    # Distance from Mars
    sb = ['./script_roadster.zsh marsDistance', 'marsDistance']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    distanceFromMarskm = float(g.stdout.strip()) \
        * float(AU_TO_KM)
    distanceFromMarsmi = float(distanceFromMarskm) \
        * float(KM_TO_MILES)

    CDATA = CDATA + '"API_mars_distance_km":' \
        + str(mars_distance_from_api_km) + ','
    CDATA = CDATA + '"API_mars_distance_mi":' \
        + str(mars_distance_from_api_mi) + ','
    CDATA = CDATA + '"LCL_mars_distance_km":' \
        + str(distanceFromMarskm) + ','
    CDATA = CDATA + '"LCL_mars_distance_mi":' \
        + str(distanceFromMarsmi) + ','

    # Orbital Speed
    sb = ['./script_roadster.zsh speed', 'speed']
    g = subprocess.run(sb, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE,
                       shell=True,  # noqa B602
                       universal_newlines=True)
    OrbitalSpeedkph = float(g.stdout.strip()) \
        * (float(60.0) * float(60.0))
    OrbitalSpeedmph = float(OrbitalSpeedkph) * float(KM_TO_MILES)

    CDATA = CDATA + '"API_speed_kph":' + str(OrbitalSpeedkph) + ','
    CDATA = CDATA + '"API_speed_mph":' + str(OrbitalSpeedmph) + ','
    CDATA = CDATA + '"LCL_speed_kph":' + str(orbital_speed_kph) + ','
    CDATA = CDATA + '"LCL_speed_mph":' + str(orbital_speed_mph) + ','

    CDATA = CDATA + '"LAST": 0}'

    return CDATA



"""Launches module

This is a module which allows wrapper access to data about
all SpaceX launches - both past and (as far as is known) in
the future

This file is imported as a module and contains the following
functions:

    * alllaunches - returns all launch information
    * latest - returns information about the latest launches
    * nextlaunch - returns information about the next launch
    * upcoming - returns information about the future launches
    * past - returns information about previous launches
    * one - returns information about a specific launch

"""
from . import urldata
from . import utils


def alllaunches(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about all SpaceX launches

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of all launch
        events respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    try:
        requestUrl = urldata.Domain.main + urldata.Domain.main_launches
    except utils.SpaceXReadTimeOut as e:
        raise e
    else:
        return utils.makeRequest(requestUrl, timeOut, parameters)


def latest(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about the latest SpaceX launches

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of the latest launch
        events respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.latest_launches
    return utils.makeRequest(requestUrl, timeOut, parameters)


def nextlaunch(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about the next SpaceX launch

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of the next launch
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.next_launches
    return utils.makeRequest(requestUrl, timeOut, parameters)


def upcoming(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about future SpaceX launches

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of the future launch
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.upcoming_launches
    return utils.makeRequest(requestUrl, timeOut, parameters)


def past(parameters='', timeOut=1):
    """

    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about past SpaceX launches

    Parameters
    ----------
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of previous launches
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain.past_launches
    return utils.makeRequest(requestUrl, timeOut, parameters)


def one(launch='', parameters='', timeOut=1):
    """
    :type launch: str
    :type parameters: Optional[str]
    :type timeOut: Optional[int]

    Returns details about a specific SpaceX launch

    Parameters
    ----------
    launch : launch ID (str)
    parameters : JSON document (str)
        JSON document containing an optional list
        of filters
    timeOut : time out in seconds

    Returns
    -------
    a string in JSON format containing details of an individual launch
        respecting any filters
    """
    utils.validateParameters(parameters, __name__, utils.func_name())
    requestUrl = urldata.Domain.main + urldata.Domain + "/" + launch
    return utils.makeRequest(requestUrl, timeOut, parameters)
